/*
 * Serial_Port.java
 *
 * Created on December 5, 2006, 9:15 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package interactive_webcam;
 import gnu.io.*;
 import java.io.*;
 import java.util.*;
/**
 *
 * @author Charles
 */
 
    
public class Serial_Port{
    
	 // DATAMEMBERS
	   SerialPort mySerialPort;
	   OutputStream out;
	   InputStream in;
	   boolean UsingSerial = false;
	   
	// DATAMEMBERS USED BY SERIALEVENT - Set to Defaults
	   char DelimiterChar = '$';
	   int NumberOfBytesToExamine = 100;
	   
	    /** Creates a new instance of Serial_Port */
	    public Serial_Port() {
	    }
	    
	   // ## NAME: ListCommPorts
	// ## DATE: 11/19/2006
	// ## AUTHOR: Charles Palen
	// ## DESCRIPTION: This function is used at the beginning of our app in order to list the available comm ports.
	// ## It returns in a comma delimited string the comm ports available for the given system.
	// ## PRECONDITIONS: None
	// ## POSTCONDITIONS: If there are comm ports, then they are returned in a comma delimited string    
	        public static String ListCommPorts(){
	           Enumeration portList = CommPortIdentifier.getPortIdentifiers();
	           String Returnstring = "";
	  		while (portList.hasMoreElements()) {
	  			CommPortIdentifier portId = (CommPortIdentifier)portList.nextElement();
	  			if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL){
	  				System.out.println(portId.getName() + " " +portId.getCurrentOwner());
	                        // Append the comm port to the return string
	                           Returnstring = Returnstring + portId.getName() + ",";
	  			}	
	  		}
	           return Returnstring;
	        }    
	        
	// ## NAME: Connect
	// ## DATE: 12/05/2006
	// ## AUTHOR: Charles Palen (palen1c@gmail.com)
	// ## DESCRIPTION: This function takes in a comm port speed and string of the comm port name.  It then sets the parameters
	// ## for the commm port connection and try's to connect.
	// ## PRECONDITIONS: A COMM PORT STREAM MUST NOT ALREADY BE OPEN, a port name, speed, and true or false if an event listener should be registered needs to be sent
	// ## POSTCONDITIONS: The port will be opened and TRUE will be returned OR, the port will not be opened and false will be returned
	      public synchronized boolean Connect (String whichPort, int whichSpeed) {
	           
	           // If the port is in use, we will wait until it is closed
	               while(this.UsingSerial == true) {
	                    try {
	                        wait();
	                    } catch (InterruptedException e){}
	               }
	               
	               
	  		try {
	  			//find the port
	  			CommPortIdentifier portId = CommPortIdentifier.getPortIdentifier(whichPort);
	  			//open the port
	  			this.mySerialPort = (SerialPort)portId.open("SerialExample" + whichPort, whichSpeed);
	                        this.UsingSerial = true;
	                        notifyAll();
	  			//configure the port
	  			try {
	                         //   mySerialPort.setSerialPortParams(whichSpeed,8,1,0);
	  					mySerialPort.setSerialPortParams(whichSpeed,
	  					mySerialPort.DATABITS_8,
	  					mySerialPort.STOPBITS_1,
	  					mySerialPort.PARITY_NONE);
	  			} catch (UnsupportedCommOperationException e){System.out.println("Error Setting Connection Parameters.  Here is the value of serialport " + mySerialPort); this.Disconnect(); return false;}
	  			//establish streams for reading and writing to the port
	  			try {
	  				this.in = this.mySerialPort.getInputStream();
	                                this.out = this.mySerialPort.getOutputStream();
	  			} catch (IOException e) { System.out.println("couldn't get streams"); return false;}
	  			
	                        //   try {
	                        //        
	  			//	mySerialPort.addEventListener(this);
	  			//	mySerialPort.notifyOnDataAvailable(true);
	  			//  } catch (TooManyListenersException e) {System.out.println("couldn't add listener");}
	                      
	  		}
	  		catch (Exception e) { System.out.println("Port in Use");return false;}
	System.out.println(whichSpeed);               
	         return true;
	  	}
	            
	// ## NAME: Disconnect
	// ## DATE: 12/05/2006
	// ## AUTHOR: Charles Palen (palen1c@gmail.com)
	// ## DESCRIPTION: This function dissconnects the comm port if one is currently connected
	// ## PRECONDITIONS: A COMM PORT STREAM MUST NOT ALREADY BE OPEN & true must be sent if we need to unregister an event listener
	// ## POSTCONDITIONS: The port will be closed so that another one can be opened or an error will be thrown
	      public synchronized void Disconnect () {
	  	    // If the port is NOT in use, we will wait until it is actual open to close it
	            // If this is indeed waiting, it means that someone called close without opening, which means thats
	            // a logic error in a higher class.
	               while(this.UsingSerial == false) {
	                    try {
	                        wait();
	                    } catch (InterruptedException e){}
	               }
	               
	          try{
	               closeIOStreams();
	               this.mySerialPort.close();
	               this.mySerialPort = null;
	               this.UsingSerial = false;
	               notifyAll();
	            } catch (Exception e) { System.out.println("Cannot close comm port");}
	      }
	   
	// ## NAME: closeIOStreams
	// ## DATE: 12/05/2006
	// ## AUTHOR: Thomas (Tom) M. Witmer 2002, 2003, 2004 function is based on code from HouseMonkey. (Modified by Charles Palen palen1c@gmail.com)
	// ## DESCRIPTION: This function closes the IO streams independently, so they can have their own try and catch statements
	// ## PRECONDITIONS: NONE
	// ## POSTCONDITIONS: The IO Streams will be closed, or exceptions thrown
	      private void closeIOStreams()
	      {
	        try
	        {
	            this.in.close();
	        }
	        catch (Exception e)
	        {
	            System.out.println( "Failed to close input stream");
	        }
	        try
	        {
	            this.out.close();
	        }
	        catch (Exception e)
	        {
	            System.out.println( "Failed to close output stream");
	        }
	    }
	    
	// ## NAME: Send_Data
	// ## DATE: 12/05/2006
	// ## AUTHOR: Thomas (Tom) M. Witmer 2002, 2003, 2004 function is based on code from HouseMonkey. (Modified by Charles Palen palen1c@gmail.com)
	// ## DESCRIPTION: Sends a string converted to a byte array of data to the comm port
	// ## PRECONDITIONS: Comm port should be open, and a string designed for output should be sent to this function
	// ## POSTCONDITIONS: Exception will be thrown, or data will be output to the port
	      protected void Send_Data(String data) throws IOException
	      {
	          if (this.mySerialPort == null)
	          {
	              System.out.println( "Serial port is closed!" );
	          } else {
	          out.write(data.getBytes());
	          out.flush();
	          }
	      }
	          
	      
	// ## NAME: Port_Status
	// ## DATE: 12/05/2006
	// ## AUTHOR: Charles Palen (palen1c@gmail.com)
	// ## DESCRIPTION: This function returns true if the port is in use, false if not in use
	// ## PRECONDITIONS: None
	// ## POSTCONDITIONS: True will be returned for in use, false for not in use
	      public boolean Get_Status () {
	          return UsingSerial;
	      }
	      
	// ## NAME: Get_Byte_Stream_As_String
	// ## DATE: 12/05/2006
	// ## AUTHOR: Charles Palen (palen1c@gmail.com)
	// ## DESCRIPTION: This function takes in a number of bytes to be examined, and a character.
	// ## The input bytes are examined for the character as the start of meaningful data and the end.
	// ## This function listens to data from serialEvent, and turns off the eventlistener once a character array is
	// ## captured from input that meets the parameters sent to this function.
	// ## PRECONDITIONS: # of bytes to be examined at a time, and start character must be passed to this.  Also, a connection should already be made to the comm port.
	// ## POSTCONDITIONS: Data is returned from this function as a string.
	      public synchronized String Get_Byte_Stream_As_String(int _NumberOfBytes, char _Delimiter) {
	       // Initializing a return character array of 200 (may need to be adjusted later)
	          String ReturnString = null;
	       
	       // Setting global datamembers for use in serialEvent
	          if (_NumberOfBytes != 0 && _Delimiter != ' '){
	             this.DelimiterChar = _Delimiter;
	             this.NumberOfBytesToExamine = _NumberOfBytes;
	          }
	          
	     // Waiting for the serial port not to be null so that we can call the function to get data from it    
	        while(this.mySerialPort == null){
	          try{
	            wait();
	          } catch (InterruptedException e){}
	        }
	      // Call the function to get data from the serial port
	         ReturnString = SerialEvent();
	               
	       
	          return ReturnString;
	      }
	      
	// ## NAME: serialEvent
	// ## DATE: 12/05/2006
	// ## AUTHOR: Charles Palen (palen1c@gmail.com)
	// ## DESCRIPTION: This function is required by gnu.io if you are going to non-abstractly implement an
	// ## event listener.  This is simply the code that runs over and over again once an event listener is
	// ## implemented on this class.  It uses "synchronized", so that I can have a parent function wait for it to notify before running the rest of the parent function.
	// ## Serial event is used in this context to read always on input from a gps unit and capture a full string of data
	// ## PRECONDITIONS: The port must be opened and have its Need_Event_Listener boolean set to ture, which will start this running
	// ## POSTCONDITIONS: Some of the data on the buffer will be examined and may be discarded or kept.
	      private String SerialEvent() {
	         // Variables that will be used
	            char ExamineLineArray[] = new char[500];
	            int ExamineCounter = 0;
	            int StartMarker = 0;
	            boolean StartMarked = false;
	            boolean EndMarked = false;
	            int EndMarker = 0;
	            boolean KeepRunning = true;
	         // The ReturnString
	            String StringtoParse = "";
	            
	         // Now the actual programming   
	            try {
	  		while ( KeepRunning == true) {
	                if(in.available() > this.NumberOfBytesToExamine){
	                                 // Determining how many bytes are actualy available (could be above 64)   
	                                    int length = in.available();      
	                                 // Initializing a byte buffer
	                                    byte c[] = new byte[length];
	                                 // Reading in what data is on the buffer
	                                    in.read(c);
	                   while(ExamineCounter < length){
	                  if (((char)c[ExamineCounter] == this.DelimiterChar) && (StartMarked ==false)){
	                     StartMarked = true;
	                     StartMarker = ExamineCounter;
	                  } else { if(((char)c[ExamineCounter] == this.DelimiterChar)&& (StartMarked == true) && (EndMarked == false)){
	                     EndMarker = ExamineCounter - 1;
	                     EndMarked = true;
	                  }
	                  }
	                  
	                  ExamineLineArray[ExamineCounter] = (char)c[ExamineCounter];     
	                  ExamineCounter = ExamineCounter + 1;
	                   } // For the ExamineCounter WHILE Loop
	                     // Resetting the examine counter
	                        ExamineCounter = 0;
	              } //For the IF in.available
	                      if(StartMarked == true && EndMarked == true){
	                         KeepRunning = false;
	                       } else {
	                    // Resetting the Start and End Markers so we don't keep a position from the last try
	                       StartMarked = false;
	                       EndMarked = false;
	                       }
	                } //For the WHILE LOOP
	            } catch(IOException e){System.out.println("Couldnt grab data from the serial port");}
	                                   
	         // Now isolating one complete NMAE sentence and looking at it closeley
	         // Example GPRMC NMAE sentence 
	         // $GPRMC,163126,V,4335.2521,N,08446.0900,W,000.0,173.2,051206,006.1,W*62

	                                      if (EndMarker > 0){
	                                        StringtoParse = String.valueOf(ExamineLineArray,StartMarker,EndMarker);

	                                   // Examining the value after the second comma to see if this is valid or invalid data
	                                   // We know this is always at position 13 in a valid GPRMC string.
	                                   // WE CURRENTLY ALLOW AN INVALID GPRMC STRING INDICATED BY 'V'
	                                   if (StringtoParse.charAt(14) == 'A' || StringtoParse.charAt(14) == 'V'){
	                                        System.out.println("Valid Data " + StringtoParse);       
	                                      // Re-awaking the thread to close the serial port and return the input from the class
	                                                 
	                                      } else {
	                                        System.out.println(StringtoParse.charAt(14) + " Invalid Data " + StringtoParse);
	                                              }
	                                        } //If statement for the End Marker

	                                        System.out.println("Start Marker " + StartMarker);
	                                        System.out.println("End Marker " + EndMarker);

	                                 return StringtoParse; 
	      }
      
}